<?xml version="1.0" encoding="US-ASCII"?>
<!--
     odsUBL2xml.xsl - convert ODS XML for UBL to labelled XML

     The result document element is <Model> with a file= being the value
     supplied at invocation as $file.

     The first row is <Headings> with a <Heading> element for each cell.  The
     ref= attribute is the text of the cell, compressed without spaces or
     oblique characters.  The element's content is the full text of the cell.

     Each remaining row is an <Item> element, with each cell being an element
     with the cell's text content.  The element's name same as the column's
     heading's ref= attribute.

     Implementation note: a column heading comprised solely of obliques is not
                          tested for and not accommodated by the algorithm.

     $Id: odsUBL2xml.xsl,v 1.6 2006/12/07 04:31:35 G. Ken Holman Exp $

     Copyright (C) 2006 National IT and Telecom Agency, Denmark
     Licensed under a Creative Commons Attribution 2.5 License. 
     See http://creativecommons.org/licenses/by/2.5/ for details
     Any derivative works must be attributed with the statement:
        "Portions originally copyright (C) 2006
         National IT and Telecom Agency, Denmark"

-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:o="urn:oasis:names:tc:opendocument:xmlns:office:1.0"
                xmlns:t="urn:oasis:names:tc:opendocument:xmlns:table:1.0"
                xmlns:x="urn:oasis:names:tc:opendocument:xmlns:text:1.0"
                xmlns:ods2xml="urn:x-ods2xml"
                exclude-result-prefixes="o t x ods2xml"
                version="2.0">

<xsl:output method="xml" indent="yes" encoding="ISO-8859-1"/>

<!--the name of the file being transformed; or some other referencing value-->
<xsl:param name="file"/>

<!--the string of characters not allowed in an element name-->
<xsl:variable name="invalidChars"
                            >!"#$%&amp;'()*+,./:;&lt;=>?@^`{|}~ </xsl:variable>

<!--find and process all rows in the document-->
<xsl:template match="/">
  <xsl:variable name="rows" select="//t:table-row"/>
  <xsl:variable name="headings" select="$rows[1]/t:table-cell"/>
  <!--start the result-->
  <Model file="{$file}">
    <!--the headings row documents all of the columns-->
    <Headings>
      <xsl:for-each select="$headings[x:p[normalize-space()]]">
        <!--one heading per column-->
        <Heading ref="{translate(normalize-space(ods2xml:allxp(.)),'/ ','')}">
          <!--the heading's value-->
          <xsl:value-of select="normalize-space(ods2xml:allxp(.))"/>
        </Heading>
      </xsl:for-each>
    </Headings>
    <!--the content rows contain all of the remaining cell content-->
    <xsl:for-each select="$rows[position()>1]
                               [normalize-space(t:table-cell[1]/x:p)]">
      <!--one item per row-->
      <Item>
        <!--one element per cell-->
        <xsl:for-each select="t:table-cell">
          <xsl:variable name="pos" 
            select="1 + count(preceding-sibling::t:table-cell) +
           sum( preceding-sibling::t:table-cell/@t:number-columns-repeated ) -
         count( preceding-sibling::t:table-cell/@t:number-columns-repeated )"/>
          <xsl:if test="$headings[$pos][x:p[normalize-space(.)]]">
            <!--name the element by the column header in first row-->
            <xsl:element name="{translate(normalize-space
                                              (ods2xml:allxp($headings[$pos])),
                                          $invalidChars,'')}">
              <!--the cell's value-->
              <xsl:value-of select="x:p"/>
            </xsl:element>
          </xsl:if>
        </xsl:for-each>
      </Item>
    </xsl:for-each>
  </Model>
</xsl:template>

<!--a given cell might have multiple paragraphs, so return concatenation-->
<xsl:function name="ods2xml:allxp">
  <xsl:param name="node"/>
  <xsl:for-each select="$node">
    <xsl:value-of select="x:p"/>
  </xsl:for-each>
</xsl:function>

</xsl:stylesheet>